/* multitalk.c
 * Client Multitalk.
 * 
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr, http://www.yeri.fr.st
 * Email: thierry.vouriot@free.fr, yeri@fr.st
 * Date: 10 novembre 2002
 */

#include "../include/multitalk.h"

/* Prototypes */
void error_quit(char *s);

int main(int argc, char **argv)
{
	struct hostent *host;
	struct sockaddr_in addr;	// adresse du serveur
	int sock;			// socket
	int port;			// port du serveur
	int nb;				// nb de char envoys
	
	char *message = MALLOCN(char, MSG_SIZE);	// buffer message
	
	fd_set rfds, afds;	// table des sockets pour la primitive select

	/* Test des arguments de la ligne de commande */
	if (argc != 3) {
		printf("Usage: multitalk nommachine port\n");
		exit(0);
	}
	
	/* Rcupre l'adresse du serveur  partir de son nom */
	host = gethostbyname(argv[1]);
	if (host == NULL) error_quit("error gethostbyname");
	
	/* Cration du socket */
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock < 0) error_quit("error socket");
	
	/* Test de la validit du port */
	port = atoi(argv[2]);
	if (port < 1024 || port > 65535) {
		printf("Numero de port invalide: 1024 < port < 65535\n");
		exit(-1);
	}
	
	/* Cration de la structure sockaddr_in (adresse du serveur) */
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);	// port
	memcpy(&addr.sin_addr, host->h_addr, sizeof(addr.sin_addr));	// copie de l'adresse IP
	memset(&(addr.sin_zero), '\0', 8);	// on met  zro le reste de la struct
	
	/* Connection au serveur */
	if (connect(sock, (struct sockaddr*) &addr, sizeof(struct sockaddr)) < 0) 
		error_quit("error connect");
	
	/* Initialisation de la table des sockets */
	FD_ZERO(&afds);
	FD_SET(sock, &afds);
	FD_SET(STDIN, &afds);	
	
	/* Boucle du client */
	while(1) {
		memcpy(&rfds, &afds, sizeof(rfds));	
		
		/* Surveillance des descripteurs en lecture */
		if ( select(sock + 1, &rfds, 0, 0, 0) == -1)
			error_quit("error select");
	
		/* lecture des messages en provenance du serveur
		 * affichage des messages sur la sortie standard
		 */
		if (FD_ISSET(sock, &rfds)) {
			if ((nb = read(sock, message, MSG_SIZE)) < 0) 
				error_quit("error read");
		
			if (nb == 0) { 
				printf("Fermeture de la connexion\n");
				close(sock);
				exit(0);
			}
			message[nb] = '\0';
			
			if (write(STDOUT, message, nb) < 0)
				error_quit("error write");
		}
		
		/* lecture de l'entre standard
		 * envoi des messages au serveur
		 */
		if (FD_ISSET(STDIN, &rfds)) {
			
			if ((nb = read(STDIN, message, MSG_SIZE)) < 0)
				error_quit("error read");
			message[nb] = '\0';
			
			if (write(sock, message, strlen(message)) < 0)
				perror("error write");
		}
	}
	
	/* Fermeture du socket */
	close(sock);
	
	return 1;
}

/* Fonction affichant un message d'erreur en fonction de la valeur
 * de errno et quitte le programme.
 */
void error_quit(char *s)
{
	perror(s);
	exit(-1);
}
