/* multitalk_srv.c
 * Serveur Multitalk.
 *
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr, http://www.yeri.fr.st
 * Email: thierry.vouriot@free.fr, yeri@fr.st
 * Date: 7 novembre 2002
 */

#include "../include/multitalk.h"
#include "userslist.h"

/* Prototypes */
bool read_client(int sock, char *msg, size_t size);
void cmd_who(Users list, int sock);
void cmd_help(int sock, bool serv);
void error_quit(char *s);

int main(int argc, char **argv)
{
	struct sockaddr_in addr;	// adresse du serveur puis adresse du client
	int serv_sock, client_sock;	// sockets
	int nb, i, fd, port, taille;	// variables
	bool ok, full;			// boolens		
	
	char *message = MALLOCN(char, MSG_SIZE);	// buffer message
	char *cmd = MALLOCN(char, CMD_SIZE);		// buffer command
	char *pseudo = MALLOCN(char, PSEUDO_SIZE+3);	// buffer pseudo
	
	fd_set rfds, afds;		// table contenant les sockets des clients
	int nfds;			// valeur (+1) du plus grand socket de la table
					// utiles pour la primitive select
	
	Users users = init_list(MAX_CLIENTS);	// table des utilisateurs enregistrs
	
	/* Test des arguments de la ligne de commande */
	if (argc != 2 && argc != 1) {
		printf("Usage: multitalk_srvr [port]\n");
		exit(0);
	}
	if (argc == 2) port = atoi(argv[1]);
	else port = 10010 + getuid();
	printf("Port = %d\n", port);
	
	/* Test de la validit du port */
	if (port < 1024 || port > 65535) {
		printf("Numero de port invalide: 1024 < port < 65535\n");
		exit(-1);
	}
	
	/* Cration du socket serveur */
	serv_sock = socket(PF_INET, SOCK_STREAM, 0);
	if (serv_sock < 0) error_quit("error socket");
	
	/* Cration de la structure sockaddr_in (adresse du serveur) */
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);	// port
	addr.sin_addr.s_addr = htonl(INADDR_ANY);	
	memset(&(addr.sin_zero), '\0', 8);	// on met  zro le reste de la struct
	
	taille = sizeof(struct sockaddr);	// taille de la structure sockaddr
	
	/* Attacher une adresse au socket */
	if (bind(serv_sock, (struct sockaddr*) &addr, taille) < 0) error_quit("error bind");
	
	/* La socket attent des connexions entrantes */
	if (listen(serv_sock, MAX_CLIENTS) < 0) error_quit("error listen");
	
	/* Initialisation de la table des sockets */
	nfds = serv_sock + 1;
	FD_ZERO(&afds);
	FD_SET(serv_sock, &afds);
	FD_SET(STDIN, &afds);
	
	printf(WELCOME);
	
	/* Boucle du serveur */
	while (1) {
		memcpy(&rfds, &afds, sizeof(rfds));
		
		/* Surveillance des descripteurs en lecture */
		if ( select(nfds, &rfds, 0, 0, 0) == -1)
			error_quit("error select");
		
		/* Nouvelle connection sur le serveur */
		if (FD_ISSET(serv_sock, &rfds)) {
			/* Connection d'un utilisateur */
			if ((client_sock = accept(serv_sock, (struct sockaddr*)&addr,  &taille) )== -1) {
				perror("error accept");
				break;
			}
			printf("Connexion client (%s)\n", inet_ntoa(addr.sin_addr));
			
			FD_SET(client_sock, &afds);	// ajout du socket du client
			if (client_sock > (nfds-1))	// maj de nfds
				nfds = client_sock + 1;
				
			// Affichage d'un message de bienvenue au client
			if (write(client_sock, WELCOME, strlen(WELCOME) + 1) < 0)
				perror("error write");
		}
		
		/* Lecture de l'entree standard (clavier */
		if (FD_ISSET(STDIN, &rfds)) {
			if ((nb = read(STDIN, message, CMD_SIZE)) == 0)
				perror("error read");
			
			// on met '\0'  la fin de la chaine et on enlve le '\n' 
			if (nb == 0) message[nb] = '\0'; else message[nb-1] = '\0';
			
			// Lecture du premier mot pour voir s'il s'agit d'une commande
			sscanf(message, "%s ", cmd);
			
			// Commande SHUTDOWN
			if (strcmp(cmd, CMD_SHUTDOWN) == 0) {
				// Message  tous les clients
				for (fd=1; fd<nfds; fd++) {
					if (fd!=serv_sock && FD_ISSET(fd, &afds))
						write(fd, SHUTDOWN, strlen(SHUTDOWN)+1);
					close(fd);
				}
				close(serv_sock);
				exit(0);
			}
			// Commande HELP
			else if (strcmp(cmd, CMD_HELP) == 0) {	
				cmd_help(STDOUT, true);
			}
			
			// Commande WHO
			else if (strcmp(cmd, CMD_WHO) == 0) {	
				cmd_who(users, STDOUT);
			}
			
			// Commande KILL
			else if (strcmp(cmd, CMD_KILL) == 0) {
				sscanf(message, KILL_PSEUDO, pseudo);
				
				if (pseudo_in_list(users, pseudo)) {	// si le pseudo appartient  la liste
					// on envoi un message  tous les utilisateurs
					sprintf(message,"Ejection de l'utilisateur (%s) par le serveur !\n", pseudo);
					for (i=0; !EOT(users[i]); i++) {
						if (users[i].socket != -1)
						if (write(users[i].socket, message, strlen(message)+1) < 0)
							perror("error write");
					}
					// et on supprime l'utilisateur
					users = del_pseudo_list(users, pseudo);
					printf("Ejection de l'utilisateur: %s\n", pseudo);
				}
				else printf("Utilisateur inconnu: %s\n", pseudo);	
			}
			
			// Vidage des buffers
			memset(message, '\0', MSG_SIZE);
			memset(cmd, '\0', CMD_SIZE);
			memset(pseudo, '\0', PSEUDO_SIZE+3);
		}
		
		/* Lecture sur tous les clients */
		for (fd =1; fd < nfds; fd ++) {
			// Client fd prt pour la lecture	
			if (fd != serv_sock && FD_ISSET(fd, &rfds)) {
				if (read_client(fd, message, MSG_SIZE) == false) {
					// suppression du client	
					close(fd);
					FD_CLR(fd, &afds);
					users = del_socket_list(users, fd);
					printf("Perte d'un client (socket %d) !!\n", fd);
				}
				else {
					// Test si le client fd appartient  la liste des utilisateurs enregistrs
					// si oui la variable pseudo est mise  jour
					ok = socket_in_list(users, pseudo, fd);
					
					// Lecture du premier mot pour voir s'il s'agit d'une commande
					sscanf(message,"%s ", cmd);
					
					// Commande CONNECT
					// Si l'utilisateur est dj connect cette commande n'a aucun effet
					if (!ok && strcmp(cmd, CMD_CONNECT) == 0) {
						sscanf(message, CONNECT_PSEUDO, pseudo);	// lecture pseudo
						
						// si il n'y pas de pseudo avec la commande CONNECT
						// on affecte le pseudo "user" + num socket
						if (strcmp(pseudo, "") == 0)
							sprintf(pseudo,"user%d",fd);
						
						// ajout de l'utilisateur dans la table si le pseudo
						// n'est pas dj utilis
						if (!pseudo_in_list(users, pseudo)) {
							users = add_list(users, pseudo, fd, &full);
							if (!full) { // liste non pleine
								printf("Nouvel utilisateur enregistre: %s\n", pseudo);
						
								// on envoie le message suivant aux autres utilisateurs
								memset(message, '\0', MSG_SIZE);
								sprintf(message, "Connexion de l'utilisateur: %s\n", pseudo);
								memset(pseudo, '\0', PSEUDO_SIZE+3);
			
								// permet de pouvoir envoyer le message
								ok = true;
							}
							else { // liste pleine
								printf(ERROR_FULL);
								if (write(fd, ERROR_FULL, strlen(ERROR_FULL)+1) < 0)
									perror("error write");
							}
						}
						// pseudo dj utilis
						else {
							if (write(fd, ERROR_PSEUDO, strlen(ERROR_PSEUDO)+1) < 0)
								perror("error write");
						}	
					}
					
					// Commande HELP
					else if (strcmp(cmd, CMD_HELP) == 0) {
						cmd_help(fd,false);
						ok = false; // pas de messages
					}
					
					// Commande WHO
					else if (strcmp(cmd,CMD_WHO) == 0) {
						cmd_who(users, fd);
						ok = false; // pas de messages
					}
					
					// Commande QUIT
					else if (strcmp(cmd,CMD_QUIT) == 0) {
						if (ok) {
							printf("Deconnexion de l'utilisateur: %s\n", pseudo);
							users = del_socket_list(users, fd); // suppression de la liste des utilisateurs
							// message  tous les autres utilisateurs
							sprintf(message, "Deconnexion de l'utilisateur: %s\n", pseudo);
							memset(pseudo, '\0', PSEUDO_SIZE);
						}
						if (write(fd, BYE, strlen(BYE)+1) < 0)
							perror("error write");
						FD_CLR(fd, &afds); // suppression ds la liste des clients
						close(fd);
					}
					
					// Envoi du message
					if (ok) {
						// on ajoute ':' aprs le pseudo
						if (strlen(pseudo) > 0) { 
							strcat(pseudo, " : ");
						}
						// envoi du message  chaque utilisateur
						// avec le nom de l'emetteur
						for (i=0; !EOT(users[i]); i++)
						if (users[i].socket!=-1 && FD_ISSET(users[i].socket, &afds)) {
							if (write(users[i].socket, pseudo, strlen(pseudo)) < 0)
								perror("error write");
							if (write(users[i].socket, message, strlen(message)+1) < 0)
								perror("error write");
						}
					}
					
					// Vidage des buffers
					memset(pseudo, '\0', PSEUDO_SIZE + 3);
					memset(cmd, '\0', CMD_SIZE);
					memset(message, '\0', MSG_SIZE);
				}
			}
		}
	}
	return 1;
}

/* Fonction qui lit les donnes sur le socket d'un client 
 * et les place dans le buffer msg 
 */
bool read_client(int sock, char *msg, size_t size) 
{
	int nb;
	
	if ((nb = read(sock, msg, size)) < 0)
		perror("error read");
	msg[nb] = '\0';
	
	if(nb > 0) {
		return true;
	}
	else return false; // retourne faux si le client n'est plus accessible
}

/* Fonction qui envoie la liste des utilisateurs sur un socket */
void cmd_who(Users list, int sock)
{
	int i;
	char *who = MALLOCN(char, PSEUDO_SIZE* (MAX_CLIENTS + 15));
	memset(who, '\0', PSEUDO_SIZE* (MAX_CLIENTS + 15));	
	strcpy(who, "\nListe des utilisateurs:\n=======================\n");
	
	for (i=0; !EOT(list[i]); i++) {
		if (list[i].socket != -1) {
			strcat(who,"- ");
			strcat(who, list[i].pseudo);
			strcat(who,"\n");
		}
	}
	strcat(who,"_______________________\n\n");
	if (write(sock, who,  PSEUDO_SIZE* (MAX_CLIENTS + 10)) <0)
		perror("error write");
	free(who);	
}

/* Fonction qui envoie la liste des commandes sur un socket, la variable
 * serv permet de definir si les commandes sont celles du serveur ou d'un client
 */
void cmd_help(int sock, bool serv)
{
	char *help = MALLOCN(char, 512);
	memset(help, '\0', 512);
	strcpy(help, "\nListe des commandes:\n================================================\n");
	strcat(help,CMD_HELP);strcat(help," : affiche la liste des commandes\n");
	strcat(help,CMD_WHO);strcat(help," : liste des utilisateurs\n");
	if (!serv) { 
		strcat(help,CMD_CONNECT);strcat(help," [pseudo] : enregistrement d'un pseudo\n");
		strcat(help,CMD_QUIT);strcat(help," : quitter le multitalk\n"); 
	}
	if (serv) { 
		strcat(help,CMD_KILL);strcat(help," pseudo : ejecte un client du serveur\n");
		strcat(help,CMD_SHUTDOWN);strcat(help," : arret du serveur\n"); 
	}
	strcat(help,"________________________________________________\n\n");
	if (write(sock, help,  512) < 0)
		perror("error write");	
	free(help);	
}

/* Fonction affichant un message d'erreur en fonction de la valeur
 * de errno et quitte le programme.
 */
void error_quit(char *s)
{
	perror(s);
	exit(-1);
}

