/* Userslist.c
 * Implantation d'une table (et des fonctions d'accs ou de modification)
 * contenant les utilisateurs du Mutlitalk.
 *
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 7 novembre 2002
 */

#include "userslist.h"

/* Initialisation de la table */
Users init_list(int size)
{
	Users list = MALLOCN(SUser, (size+1));
	int i;
	for (i=0; i<size; i++) {
		list[i].socket = -1;
		list[i].pseudo = MALLOCN(char, PSEUDO_SIZE);
		memset(list[i].pseudo, '\0', PSEUDO_SIZE);	
	}
	list[size].socket = -99;
			
	return list;
}

/* Ajout d'un utilisateur dans la table,
 * la variable full est initialis  "vrai" si
 * la table est pleine et que l'ajout est impossible
 */
Users add_list(Users list, char *pseu, int sock, bool *full)
{
	int i;
	for (i=0; !EOT(list[i]); i++) {
		if (list[i].socket == -1) {
			list[i].socket = sock;
			strcpy(list[i].pseudo, pseu);
			*full = false;
			return list;
		}
	}
	*full = true;
	
	return list;
}

/* Supprime un utilisateur en fonction de son socket */
Users del_socket_list(Users list, int s)
{
	int i;
	for (i=0; !EOT(list[i]); i++) {
		if (list[i].socket == s) {
			list[i].socket = -1;
			memset(list[i].pseudo, '\0', PSEUDO_SIZE);
			break;
		}
	}
	
	return list;
}

/* Supprime un utilisateur en fonction de son pseudo */
Users del_pseudo_list(Users list, char *pseu)
{
	int i;
	for (i=0; !EOT(list[i]); i++) {
		if (list[i].socket != -1 && strcmp(list[i].pseudo, pseu) == 0) {
			list[i].socket = -1;
			memset(list[i].pseudo, '\0', PSEUDO_SIZE);
			break;
		}
	}
	
	return list;
}

/* Test si un socket appartient  la table, si oui la valeur
 * "vrai" est renvoye et la variable pseu est remplie avec le
 * le pseudo correspondant au socket
 */
bool socket_in_list(Users list, char *pseu, int sock)
{
	int i;
	for (i=0; !EOT(list[i]); i++) {
		if (list[i].socket == sock) {
			strcpy(pseu, list[i].pseudo);
			return true;
		}
	}
	
	return false;
}

/* Test si un pseudo est prsent dans la table */
bool pseudo_in_list(Users list, char *pseu)
{
	int i;
	for (i=0; !EOT(list[i]); i++) {
		if (list[i].socket != -1 && strcmp(list[i].pseudo, pseu) == 0) {
			return true;
		}
	}
	
	return false;
}

/* Affiche la table sur la sortie standard */
void aff_list(Users list)
{
	int i;
	for (i=0; !EOT(list[i]); i++) {
		if (list[i].socket != -1) {
			printf("- %s\n",list[i].pseudo);
		}
	}
}
